/*
 connect.c : GMasqDialer

    Copyright (C) 1998-1999 Timo Sirainen
    Copyright (C) 2000-2002 David Pinson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gmasqdialer.h"

gint connection_timetag;
time_t connected_at, server_time_diff;

static GList *connlist;
static gint init_state;
static gint selected;

void callback_checkauth(CALLBACK_DATA *data)
{
    while (gtk_events_pending())
	gtk_main_iteration();
    
    if (data->status == 0) return;
    
    if (strncmp(data->u.text, "AUTHORIZED:", 11) == 0)
	auth_status = 0;
    else
	auth_status = 1;	
}

void callback_connectiontime(CALLBACK_DATA *data)
{
    init_state = 1;

    if (data->status == 0) {
        /* Old server, didn't understand us.. */
        connected_at = (time_t) 0;
        callback_err = 1;
        return;
    }

    connected_at = (time_t) data->u.digit;
    
    /* if server tells us 0, it is definitely wrong, use NOW so
     * at least things work correctly in the client 
     */
    //if (connected_at == 0)
    //    connected_at = time(NULL);

    /* Logic error, 0 means we have no connection, arg if the server
       also tells us this when the connection is up?
       Need to fix stuff elsewhere so only UP && 0 gets hacked here....
    */

    callback_err = 0;
}

static void callback_connectionlist(CALLBACK_DATA *data)
{
    if (data->status == 0) {
        /* Server is really messed up if we ever get here :) */
        callback_err = 1;
        return;
    }

    callback_err = 0;
    if (data->u.list.type == LIST_END) {
        /* List finished */
        init_state = 2;
        return;
    }

    if (data->u.list.type == LIST_ITEM) {
	if ((autodial_isp != NULL && g_strcasecmp(data->u.list.data, autodial_isp) == 0) ||
	    (selected == -1 && g_strcasecmp(data->u.list.data, defaultisp) == 0))
	{
            /* this is the autodial / last dialed isp, set it as default */
	    selected = g_list_length(connlist);
	}
	connlist = g_list_append(connlist, g_strdup(data->u.list.data));
    }
}

static GtkWidget *add_new_connection(gchar *name, 
				     gchar *info, 
				     gchar *addrs, 
				     gchar *users)
{
    GString *str;
    GtkWidget *con_item;

    str = g_string_new(name);
    if (info != NULL) g_string_append_printf(str, " (%s)", info);

    con_item = gtk_menu_item_new_with_label(str->str);
    gtk_menu_shell_append(GTK_MENU_SHELL(con_menu), con_item);
    gtk_widget_show(con_item);

    g_string_free(str, TRUE);

    g_object_set_data(G_OBJECT(con_item), "name", name);
    if (info != NULL) g_object_set_data(G_OBJECT(con_item), "info", info);
    if (addrs != NULL) g_object_set_data(G_OBJECT(con_item), "addrs", addrs);
    if (users != NULL) g_object_set_data(G_OBJECT(con_item), "users", users);

    return con_item;
}

static void callback_connectioninfo(CALLBACK_DATA *data)
{
    static gchar *name, *info, *addrs, *users;
    gchar *str;

    if (data->status == 0)
    {
        /* Old server, didn't understand us.. */
        callback_err = 1;
        return;
    }

    callback_err = 0;
    switch (data->u.list.type)
    {
        case LIST_START:
            name = g_strdup(connlist->data);
            info = addrs = users = NULL;
            break;

        case LIST_END:
            g_free(connlist->data);
            connlist->data = add_new_connection(name, info, addrs, users);

            if (connlist->next != NULL)
                mserver_connectinfo(connlist->next->data, callback_connectioninfo);

            if (connlist->next == NULL || callback_err)
                init_state = 4;
            else
                connlist = connlist->next;

            break;

        case LIST_ITEM:
            str = data->u.list.data;
            if (strncmp(str, "INFO: ", 6) == 0)
                info = g_strdup(str+6);
            else if (strncmp(str, "VALID ADDERSSES: ", 17) == 0)
                addrs = g_strdup(str+17);
            else if (strncmp(str, "VALID USERS: ", 13) == 0)
                users = g_strdup(str+13);
            break;
    }
}

static int timefunc_initialize(void)
{
    GtkWidget *con_item;

    /* First wait for READY.. */
    if (!ready) return 1;

    switch (init_state)
    {
        case 0:
            mserver_connectiontime(callback_connectiontime);
            break;
	case 1:
	    selected = -1;
            mserver_list_connections(callback_connectionlist);
            break;
        case 2:
            connlist = g_list_first(connlist);
            if (connlist != NULL)
                mserver_connectinfo((gchar *) connlist->data, 
				    callback_connectioninfo);
            if (connlist == NULL || callback_err)
            {
                GList *tmp;

                for (tmp = g_list_first(connlist); tmp != NULL; tmp = tmp->next)
                {
                    GtkWidget *li;

                    li = add_new_connection(tmp->data, NULL, NULL, NULL);

                    g_free(tmp->data);
                    tmp->data = li;
                }
                init_state = 3;
            }
            break;
        case 3:
            mserver_status(callback_status);
            init_state = 4;
            break;
        case 4:
	    if (selected != 1)
		gtk_option_menu_set_history(GTK_OPTION_MENU(con_opt), selected);

            connection_timetag = gtk_timeout_add(1000, 
						 (GtkFunction)timefunc_running,
						 NULL);
            return 0;
    }
    return 1;
}

void menu_connect(void)
{
    if (mserver_connected())
    {
        masqerror(_("You are already connected"));
        return;
    }
    if (*masqhost == '\0')
    {
        masqerror(_("No server address set - set it from preferences"));
        return;
    }

    if (mserver_connect(masqhost, masqport))
    {
        set_dial_sensitive(TRUE, FALSE);
        set_connection_sensitive(TRUE);
        init_state = 0; 
	connlist = NULL;
        connection_timetag = gtk_timeout_add(10, 
					     (GtkFunction) timefunc_initialize,
					     NULL);

	if (autosenduser) {
	    if (username != NULL && *username != '\0' && userpass != NULL && *userpass != '\0')
	    {
		mserver_setuser(username, NULL);
		mserver_setpass(userpass, NULL);
		user_sent = TRUE;
		
		/*	mserver_checkauth(callback_checkauth);*/
		
	    } else {
		menu_changeuser();
	    }
	}
    }
}
