/*
 conninfo.c : GMasqDialer

    Copyright (C) 1998-1999 Timo Sirainen
    Copyright (C) 2002      David Pinson


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gmasqdialer.h"

/*
  A 3 column line in a table

  Makes a HIG block

  +--------------------------+
  |<b>Title</b>  |           | leader != NULL
  +--------------------------+
  |  |label str  | gtk entry | leader == NULL
  +--------------------------+

*/
static void create_entry(GtkWidget *table,
			 gchar *leader,
			 gchar *title, 
			 gchar *field, 
			 gint pos)
{
    GtkWidget *label;
    GtkWidget *entry;

    if (leader != NULL) {
	label = gtk_hig_label_new(leader);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 2, pos, pos+1);
    } else {
	label = gtk_hig_label_new("    ");
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, pos, pos+1);

	label = gtk_hig_label_new(title);
	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, pos, pos+1);

	entry = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
	if (field != NULL)
	{
	    gtk_entry_set_text(GTK_ENTRY(entry), field);
	    g_free(field); /* why? */
	}
	gtk_table_attach_defaults(GTK_TABLE(table), entry, 2, 3, pos, pos+1);

    }

}


static void conninfo_dialog(gchar *name, 
			    gchar *info, 
			    gchar *script, 
			    gchar *kill, 
			    char *addrs, 
			    char *users)
{
    GtkWidget *dialog;
    GtkWidget *table;

    dialog = gtk_hig_dialog_new(_("Connection Information"));

    g_signal_connect(GTK_OBJECT(dialog), "delete_event",
		     GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);

    gtk_dialog_add_button(GTK_DIALOG(dialog), 
			  GTK_STOCK_CLOSE, 
			  GTK_RESPONSE_CLOSE);

    gtk_dialog_set_default_response (GTK_DIALOG(dialog),
				     GTK_RESPONSE_CLOSE);

    g_signal_connect_swapped (GTK_OBJECT (dialog), "response", 
			      G_CALLBACK (gtk_widget_destroy),
			      GTK_OBJECT (dialog));

    table = gtk_table_new(10, 3, FALSE);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), 
		       table, 
		       TRUE, 
		       TRUE, 
		       0);

    gtk_table_set_row_spacings(GTK_TABLE(table), 6);
    gtk_table_set_col_spacings(GTK_TABLE(table), 12);
    gtk_container_set_border_width (GTK_CONTAINER(table), 6);

    create_entry(table, _("<b>Connection</b>"), "", NULL, 0);
    create_entry(table, NULL, _("Name"), name, 1);
    create_entry(table, NULL, _("Description"), info, 2);

    create_entry(table, _("<b>Server Security</b>"), "", NULL, 3);
    create_entry(table, NULL, _("Valid IP addresses"), addrs, 4);
    create_entry(table, NULL, _("Valid users"), users, 5);

    create_entry(table, _("<b>Scripts</b>"), "", NULL, 6);
    create_entry(table, NULL, _("Dial"), script, 7);
    create_entry(table, NULL, _("Hang up"), kill, 8);

    gtk_widget_show_all(dialog);
}

static void callback_conninfo(CALLBACK_DATA *data)
{
    static gchar *name, *info, *script, *kill, *addrs, *users;

    if (data->status == 0) return;

    switch (data->u.list.type)
    {
        case LIST_START:
            name = info = script = kill = addrs = users = NULL;
            break;
        case LIST_END:
            conninfo_dialog(name, info, script, kill, addrs, users);
            break;
        case LIST_ITEM:
            if (strncmp(data->u.list.data, "NAME: ", 6) == 0)
                name = g_strdup(data->u.list.data+6);
            else if (strncmp(data->u.list.data, "INFO: ", 6) == 0)
                info = g_strdup(data->u.list.data+6);
            else if (strncmp(data->u.list.data, "SCRIPT: ", 8) == 0)
                script = g_strdup(data->u.list.data+8);
            else if (strncmp(data->u.list.data, "KILL: ", 6) == 0)
                kill = g_strdup(data->u.list.data+6);
            else if (strncmp(data->u.list.data, "VALID ADDRESSES: ", 17) == 0)
                addrs = g_strdup(data->u.list.data+17);
            else if (strncmp(data->u.list.data, "VALID USERS: ", 13) == 0)
                users = g_strdup(data->u.list.data+13);
            break;
    }
}

void menu_conninfo(void)
{
    GtkObject *object;
    GtkWidget *menu_item;
    gchar *name;

    menu_item = gtk_menu_get_active(GTK_MENU(con_menu));
    name = g_object_get_data(G_OBJECT(menu_item), "name");

    if (!mserver_connected()) return;

    if (name == NULL) {
        masqerror(_("You haven't selected a connection."));
        return;
    }

    mserver_connectinfo(name, callback_conninfo);
}
