/*
 gmasqdialer.c : GMasqDialer

    Copyright (C) 1998-1999 Timo Sirainen
    Copyright (C) 2000-2002 David Pinson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gmasqdialer.h"

#ifdef BUILD_APPLET
#include <applet-widget.h>
gboolean applet_running;
gboolean main_running;
static gboolean panel_vertical;
static gboolean panel_tiny;
static gint panel_size;

#endif

gchar *autodial_isp;
gboolean callback_err;
GtkWidget *window;
//GtkWidget *combo, *combo_entry;
GtkWidget *con_opt, *con_menu, *con_item;
GtkWidget *statusbar, *connlabel, *serverlog_text, *serverlog_scroll;

static GtkWidget *anim_da;
static GdkPixmap *anim_pixmap;
static GdkBitmap *anim_mask;
static gint anim_frame, anim_frames, anim_tag;

static GtkWidget *dialbutton, *hangbutton;

#ifdef BUILD_APPLET
GtkWidget *appletlabel;

static GtkWidget *applet;
static gboolean mainwin_hidden;

char tooltipstring[20];
int settip;
#endif

static GtkItemFactoryEntry menu_items[] = {
    {"/_Server",                  NULL,         NULL,                 0, "<Branch>"},
    {"/_Server/_Connect",         NULL,         menu_connect,         0, NULL},
    {"/_Server/_Disconnect",      NULL,         menu_disconnect,      0, NULL},
    {"/_Server/sep1",             NULL,         NULL,                 0, "<Separator>"},
    {"/_Server/_Server Info",     NULL,         menu_serverinfo,      0, NULL},
    {"/_Server/_Change User",     NULL,         menu_changeuser,      0, NULL},
    {"/_Server/_Preferences",     NULL,         menu_setup,           0, NULL},
    {"/_Server/sep1",             NULL,         NULL,                 0, "<Separator>"},
    {"/_Server/_Quit",            "<control>Q", menu_quit,            0, NULL},
    {"/_Connection",              NULL,         NULL,                 0, "<Branch>"},
    {"/_Connection/_Dial",        "<control>D", menu_dial,            0, NULL},
    {"/_Connection/_Hang up",     "<control>H", menu_hangup,          0, NULL},
    {"/_Connection/_Information", NULL,         menu_conninfo,        0, NULL},
    {"/_Help",                    NULL,         NULL,                 0, "<Branch>"},
    {"/_Help/_About",             NULL,         menu_about,           0, NULL},
};

static GtkItemFactory *item_factory = NULL;

/* Returns a menubar widget made from the above menu */
void get_menubar_menu( GtkWidget  *window,
		       GtkWidget **menubar)
{
    //GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof(menu_items) / sizeof (menu_items[0]);

  /* Make an accelerator group (shortcut keys) */
  accel_group = gtk_accel_group_new ();

  /* Make an ItemFactory (that makes a menubar) */
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
                                       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  /* Finally, return the actual menu bar created by the item factory. */
  if (menubar)
      *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}

/** report errors returned from the masqdialer server
 *
 */
void masqerror(char *str)
{
    GtkWidget *dialog;

    dialog = gtk_message_dialog_new (GTK_WINDOW(window),
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_MESSAGE_ERROR,
				     GTK_BUTTONS_CLOSE,
				     "%s",
				     str);

    g_signal_connect_swapped (GTK_OBJECT (dialog), "response", 
			      G_CALLBACK (gtk_widget_destroy), 
			      GTK_OBJECT (dialog));

    gtk_widget_show_all(dialog);
}

#ifdef BUILD_APPLET
static void applet_connected(void)
{
    applet_widget_register_stock_callback(APPLET_WIDGET(applet),
                                          "hangup",
                                          GNOME_STOCK_MENU_PREF,
                                          _("Hang Up"),
                                          (AppletCallbackFunc) menu_hangup,
                                          NULL);

    if (mainwin_hidden)
	applet_hidemain();
    else
	applet_showmain();
}

static void applet_disconnected(void)
{
    applet_widget_register_stock_callback(APPLET_WIDGET(applet),
                                          "dial",
                                          GNOME_STOCK_MENU_PREF,
                                          _("Dial"),
                                          (AppletCallbackFunc) menu_dial,
                                          NULL);

    if (mainwin_hidden)
	applet_hidemain();
    else
	applet_showmain();
}
#endif

void set_dial_sensitive(int dial, int hangup)
{

    GtkWidget *menu;

    if ((menu = gtk_item_factory_get_widget(item_factory, "/Connection/Dial")) != NULL)
	gtk_widget_set_sensitive(GTK_WIDGET(menu), dial);
    if ((menu = gtk_item_factory_get_widget(item_factory, "/Connection/Hang up")) != NULL)
	gtk_widget_set_sensitive(GTK_WIDGET(menu), hangup);

    gtk_widget_set_sensitive(dialbutton, dial);
    gtk_widget_set_sensitive(hangbutton, hangup);

#ifdef BUILD_APPLET
    if (applet_running)
    {
	applet_widget_unregister_callback(APPLET_WIDGET(applet), "dial");
	applet_widget_unregister_callback(APPLET_WIDGET(applet), "hangup");
	if (hangup)
	    applet_connected();
	else if (dial)
	    applet_disconnected();
    }
#endif
}

void set_connection_sensitive(gint connected)
{

    GtkWidget *menu;

    if ((menu = gtk_item_factory_get_widget(item_factory, "/Server/Connect")) != NULL)
	gtk_widget_set_sensitive(GTK_WIDGET(menu), !connected);
    if ((menu = gtk_item_factory_get_widget(item_factory, "/Server/Disconnect")) != NULL)
	gtk_widget_set_sensitive(GTK_WIDGET(menu), connected);
    if ((menu = gtk_item_factory_get_widget(item_factory, "/Server/Server Info")) != NULL)
	gtk_widget_set_sensitive(GTK_WIDGET(menu), connected);
    if ((menu = gtk_item_factory_get_widget(item_factory, "/Server/Change User")) != NULL)
	gtk_widget_set_sensitive(GTK_WIDGET(menu), connected);
    if ((menu = gtk_item_factory_get_widget(item_factory, "/Connection/Information")) != NULL)
	gtk_widget_set_sensitive(GTK_WIDGET(menu), connected);

}

gint timefunc_running(void)
{
    static gint n = 0;
    static gint netload = 1;

    if (++n == 3)
    {
        /* check if connection is still on */
        n = 0;
        if (!ready) return 1;
        mserver_status(callback_status);
    }

    if (connected_at > 0)
    {
        /* update connect time */
        time_t conntime;
        GString *str;

	conntime = time(NULL)-connected_at+server_time_diff;
        str = g_string_new(NULL);
        g_string_printf(str, "%02ld:%02ld:%02ld", conntime / 3600, (conntime / 60) % 60, conntime % 60);
#ifdef BUILD_APPLET
        if (appletlabel != NULL) gtk_label_set(GTK_LABEL(appletlabel), str->str);
	snprintf(tooltipstring,18, "%s %d",str->str,settip);
	
	if (settip==1) {
	    /*applet_widget_set_widget_tooltip (APPLET_WIDGET (applet), GTK_WIDGET(table), tooltipstring);*/
	    settip=0;
	    /*applet_widget_set_tooltip (APPLET_WIDGET (applet), tooltipstring);*/
	}
#endif
        g_string_prepend(str, _("Connected "));
        gtk_label_set_text(GTK_LABEL(connlabel), str->str);
        g_string_free(str, TRUE);

        if (ready && netload)
        {
            mserver_netload(callback_netload);
            if (callback_err)
            {
                /* Too old server for this.. */
                netload = 0;
            }
        }
    }
    else if (maxcps != -1)
    {
	/* connected, get connection time.. */
        mserver_connectiontime(callback_connectiontime);
    }

    return 1;
}

void menu_quit(void)
{
    if (!mserver_connected())
        gtk_main_quit();
    else
    {
        quit_after_disconnect = TRUE;
        menu_disconnect();
    }
}

void menu_about(void)
{
    GtkWidget *about;
    GtkWidget *label;

    about = gtk_hig_dialog_new_with_buttons( "About "PACKAGE, 
					     GTK_WINDOW(window),
					     GTK_DIALOG_DESTROY_WITH_PARENT, 
					     GTK_STOCK_OK, 
					     GTK_RESPONSE_NONE,
					     NULL);
    
    g_signal_connect_swapped (GTK_OBJECT (about), "response", 
			      G_CALLBACK (gtk_widget_destroy),
			      GTK_OBJECT (about));

    /* Show logo here.... */

    label = gtk_hig_label_new("<span weight='bold' size='x-large'>"PACKAGE" "VERSION"</span>");
    gtk_alignment_set(GTK_ALIGNMENT(label), 0.5, 0.5, 0, 1);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox), label, TRUE, TRUE, 0);

    label = gtk_hig_label_new("Gtk2/GNOME Masqdialer Client");
    gtk_alignment_set(GTK_ALIGNMENT(label), 0.5, 0.5, 0, 1);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox), label, TRUE, TRUE, 0);

    label = gtk_hig_label_new("<span size='small'>(C) 1998-1999 Timo Sirainen\n(C) 2000-2002 David Pinson</span>");
    gtk_alignment_set(GTK_ALIGNMENT(label), 0.5, 0.5, 0, 1);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox), label, TRUE, TRUE, 0);

    gtk_widget_show_all(about);
}

#ifdef BUILD_APPLET
static void panel_redraw(GtkWidget *applet)
{
    GtkWidget *table, *frame;

    /* destroy old netload pixmaps */
    frame = gtk_object_get_data(GTK_OBJECT(applet), "outputframe");
    if (frame != NULL)
    {
	netload_pixmap_destroy(&paneloutput);
	gtk_widget_destroy(frame);
    }
    frame = gtk_object_get_data(GTK_OBJECT(applet), "inputframe");
    if (frame != NULL)
    {
	netload_pixmap_destroy(&panelinput);
	gtk_widget_destroy(frame);
    }

    table = gtk_object_get_data(GTK_OBJECT(applet), "table");
    if (panel_show_output)
    {
        frame = gtk_frame_new(NULL);
	gtk_object_set_data(GTK_OBJECT(applet), "outputframe", frame);
        gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
        if (panel_size >= 48)
            gtk_table_attach_defaults(GTK_TABLE(table), frame, 1, 2, 1, 2);
        else
	{
	    if (panel_vertical)
		gtk_table_attach_defaults(GTK_TABLE(table), frame, 0, 1, 3, 4);
	    else
		gtk_table_attach_defaults(GTK_TABLE(table), frame, 3, 4, 0, 1);
	}
		
	if (panel_size == 24)
	    netload_pixmap_new(frame, &paneloutput, 18, 18);
	else if (panel_size == 36)
	    netload_pixmap_new(frame, &paneloutput, 18, 28);
	else if (panel_size == 48)
	    netload_pixmap_new(frame, &paneloutput, 19, 19);
	else if (panel_size == 64)
	    netload_pixmap_new(frame, &paneloutput, 27, 40);
	else
	    netload_pixmap_new(frame, &paneloutput, 35, 55);
	gtk_widget_show_all(frame);
    }

    if (panel_show_input)
    {
        frame = gtk_frame_new(NULL);
	gtk_object_set_data(GTK_OBJECT(applet), "inputframe", frame);
        gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
        if (panel_size >= 48)
            gtk_table_attach_defaults(GTK_TABLE(table), frame, 0, 1, 1, 2);
	else
	{
	    if (panel_vertical)
		gtk_table_attach_defaults(GTK_TABLE(table), frame, 0, 1, 2, 3);
	    else
		gtk_table_attach_defaults(GTK_TABLE(table), frame, 2, 3, 0, 1);
	}

	if (panel_size == 24)
	    netload_pixmap_new(frame, &panelinput, 18, 18);
	else if (panel_size == 36)
	    netload_pixmap_new(frame, &panelinput, 18, 28);
	else if (panel_size == 48)
	    netload_pixmap_new(frame, &panelinput, 19, 19);
	else if (panel_size == 64)
	    netload_pixmap_new(frame, &panelinput, 27, 40);
	else
	    netload_pixmap_new(frame, &panelinput, 35, 55);
	gtk_widget_show_all(frame);
    }
}

static void applet_change_orient(GtkWidget *applet, 
				 PanelOrientType orient, 
				 gpointer data)
{
    panel_vertical = orient == ORIENT_LEFT || orient == ORIENT_RIGHT;
    
    if (panel_size < 48 && panel_vertical == TRUE)
	    gtk_widget_hide(appletlabel);
    else
	    gtk_widget_show(appletlabel);
	    
    panel_redraw(applet);
}

#ifdef HAVE_PANEL_PIXEL_SIZE
static void applet_change_pixel_size(GtkWidget *applet, int size, gpointer data)
{
    panel_tiny = FALSE;
    switch (size)
    {
        case PIXEL_SIZE_TINY:
	    panel_size = 24;
	    panel_tiny = TRUE;
	    if (panel_vertical == TRUE) 
		    gtk_widget_hide(appletlabel);
	    else
		    gtk_widget_show(appletlabel);
            break;
	case PIXEL_SIZE_SMALL:
	    panel_size = 36;
	    if (panel_vertical == TRUE) 
		    gtk_widget_hide(appletlabel);
	    else
		    gtk_widget_show(appletlabel);
	    break;
        case PIXEL_SIZE_STANDARD:
            panel_size = 48;
	    gtk_widget_show(appletlabel);
            break;
        case PIXEL_SIZE_LARGE:
            panel_size = 64;
	    gtk_widget_show(appletlabel);
            break;
        case PIXEL_SIZE_HUGE:
            panel_size = 80;
	    gtk_widget_show(appletlabel);
            break;
    }
    
    panel_redraw(applet);
}
#endif

void applet_showmain(void)
{
    gtk_widget_show(window);
    mainwin_hidden = FALSE;

    applet_widget_unregister_callback(APPLET_WIDGET(applet), "hidewindow");
    applet_widget_unregister_callback(APPLET_WIDGET(applet), "showwindow");
    applet_widget_register_stock_callback(APPLET_WIDGET(applet),
                                          "hidewindow",
                                          GNOME_STOCK_MENU_CLOSE,
                                          _("Hide main window"),
                                          (AppletCallbackFunc) applet_hidemain,
                                          NULL);
}

static void applet_togglemain( GtkWidget *widget, GdkEventButton *ev, gpointer data )
{
    if( ev == NULL )
	return;

    if( ev->button != 1 || ev->type != GDK_BUTTON_PRESS )
	return;

    if (mainwin_hidden == FALSE)
        applet_hidemain();
    else
        applet_showmain();
}

gint applet_hidemain(void)
{
    gtk_widget_hide(window);
    mainwin_hidden = TRUE;

    applet_widget_unregister_callback(APPLET_WIDGET(applet), "hidewindow");
    applet_widget_unregister_callback(APPLET_WIDGET(applet), "showwindow");
    applet_widget_register_stock_callback(APPLET_WIDGET(applet),
                                          "showwindow",
                                          GNOME_STOCK_MENU_NEW,
                                          _("Show main window"),
                                          (AppletCallbackFunc) applet_showmain,
                                          NULL);
    return TRUE;
}

static gint applet_save_session(void)
{
    return FALSE;
}

/* Work around disappearing tool tips */
void applet_mouse_entered(GtkWidget *applet)
{
    g_print("Have entered the applet: %s\n", tooltipstring);
    settip=1;
}

void create_applet(void)
{
    GtkWidget *frame, *table;

    applet = applet_widget_new(PACKAGE);
    if (applet == NULL)
        g_error("Can't create applet, try --no-applet option.\n");

    gtk_widget_set_events( applet, gtk_widget_get_events( applet ) |
	               GDK_BUTTON_PRESS_MASK | GDK_EXPOSURE_MASK );
    gtk_signal_connect(GTK_OBJECT(applet), "destroy",
                       (GtkSignalFunc) menu_quit, NULL);
    gtk_signal_connect(GTK_OBJECT(applet), "button_press_event",
                       (GtkSignalFunc) applet_togglemain, NULL);


    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);

    applet_widget_set_tooltip (APPLET_WIDGET (applet), "Gmasqdialer");
    
    applet_widget_add(APPLET_WIDGET (applet), frame);

    table = gtk_table_new(2, 2, FALSE);
    gtk_object_set_data(GTK_OBJECT(applet), "table", table);
    gtk_container_add(GTK_CONTAINER(frame), table);

    if (!panel_show_label)
        appletlabel = NULL;
    else
    {
        appletlabel = gtk_label_new(_("No conn"));
        /*applet_widget_set_tooltip (APPLET_WIDGET (applet), _("Disconnected"));*/
        /*gtk_table_attach_defaults(GTK_TABLE(table), appletlabel, 0, 2, 0, 1);*/
        gtk_table_attach(GTK_TABLE(table), 
			 appletlabel, 0, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
    }
    gtk_widget_show_all(applet);

    /* work around disapperaing tooltips */
    gtk_signal_connect(GTK_OBJECT(applet), "enter_notify_event",
		       GTK_SIGNAL_FUNC(applet_mouse_entered), NULL);

    gtk_signal_connect(GTK_OBJECT(applet), "save_session",
                       GTK_SIGNAL_FUNC(applet_save_session), NULL);

    gtk_signal_connect(GTK_OBJECT(applet), "change_orient",
                       GTK_SIGNAL_FUNC(applet_change_orient), NULL);
#ifdef HAVE_PANEL_PIXEL_SIZE
    gtk_signal_connect(GTK_OBJECT(applet), "change_pixel_size",
		       GTK_SIGNAL_FUNC(applet_change_pixel_size), NULL);
#endif

    applet_widget_register_stock_callback(APPLET_WIDGET(applet),
                                          "about",
                                          GNOME_STOCK_MENU_ABOUT,
                                          _("About"),
                                          (AppletCallbackFunc) menu_about,
                                          NULL);

    applet_widget_register_stock_callback(APPLET_WIDGET(applet),
                                          "preferences",
                                          GNOME_STOCK_MENU_PREF,
                                          _("Preferences"),
                                          (AppletCallbackFunc) menu_setup,
					  NULL);

}
#endif

int main (int argc, char *argv[])
{
#ifdef HAVE_GNOME
    struct poptOption options[] = {
        POPT_AUTOHELP
#ifdef HAVE_GNOME_PANEL
	{ "no-applet", '\0', POPT_ARG_NONE, NULL, 0, NULL, N_("Don't run as applet") },
	{ "no-mainwin", '\0', POPT_ARG_NONE, NULL, 0, NULL, N_("Only run applet") },
#endif
	{ "dial", 'd', POPT_ARG_STRING, &autodial_isp, 0, N_("Automatically dial to ISP"), N_("ISP") },
        { NULL, '\0', 0, NULL }
    };
#endif
    GtkWidget *table, *mainvbox, *vbox, *hbox;
    GtkWidget *frame, *label, *popup;

    GtkWidget *menubar;
#ifdef BUILD_APPLET
    gint num;
#endif

#ifdef ENABLE_NLS
    /* initialize the i18n stuff */
    bindtextdomain(PACKAGE, LOCALEDIR);
    textdomain(PACKAGE);
#endif
    /*autodial_isp = NULL;*/

#ifdef BUILD_APPLET
    applet_running = TRUE;
    main_running = TRUE;
    for (num = 1; num < argc; num++)
    {
        if (strcmp(argv[num], "--no-applet") == 0)
        {
        	applet_running = FALSE;
        	break;
        }
        if (strcmp(argv[num], "--no-mainwin") == 0)
        {
        	applet_running = TRUE;
		main_running = FALSE;
        	break;
        }

    }

    if (applet_running)
        applet_widget_init(PACKAGE, VERSION, argc, argv, options, 0, NULL);
    else
        gnome_init_with_popt_table(PACKAGE, VERSION, argc, argv, options, 0, NULL);
#elif defined (HAVE_GNOME)
    gnome_init_with_popt_table(PACKAGE, VERSION, argc, argv, options, 0, NULL);
#else
    gtk_init(&argc, &argv);
#endif

    setup_init();
    protocol_init();
    netload_init();

    window = gnome_app_new(PACKAGE, PACKAGE);
    gtk_window_set_resizable(GTK_WINDOW(window), FALSE);

#ifdef APPLET
    if (applet_running)
    {
        create_applet();
        gtk_signal_connect_object(GTK_OBJECT (window), "delete_event",
                                  GTK_SIGNAL_FUNC(applet_hidemain), GTK_OBJECT(window));
    }
    else
#endif
	g_signal_connect_object(GTK_OBJECT (window), "delete_event",
				GTK_SIGNAL_FUNC(menu_quit), GTK_OBJECT(window), G_CONNECT_AFTER);


    mainvbox = gtk_vbox_new(FALSE, 0);
    gnome_app_set_contents(GNOME_APP(window), mainvbox);

    /* menubar / popup menu */
    get_menubar_menu(window, &menubar);
    gtk_box_pack_start (GTK_BOX(mainvbox), menubar, FALSE, TRUE, 0);

    /* in the middle goes everything else ... */
    table = gtk_table_new(4, 3, FALSE);
    gtk_container_set_border_width(GTK_CONTAINER(table), 6);
    gtk_table_set_col_spacings(GTK_TABLE(table), 6);
    gtk_table_set_row_spacings(GTK_TABLE(table), 6);
    gtk_box_pack_start(GTK_BOX(mainvbox), table, FALSE, TRUE, 0); 

    /* status bar */
    statusbar = gtk_statusbar_new();
    gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(statusbar), FALSE);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, "");
    gtk_box_pack_start(GTK_BOX(mainvbox), statusbar, FALSE, FALSE, 0);

    /* vbox to hold connect combo and connected labels */
    vbox = gtk_vbox_new(FALSE, 6);
    gtk_table_attach_defaults(GTK_TABLE(table), vbox, 0, 1, 0, 2);

    /* connect box */
    con_opt = gtk_option_menu_new();
    gtk_widget_set_size_request(GTK_WIDGET(con_opt), 180, -1);
    con_menu = gtk_menu_new();

    gtk_option_menu_set_menu(GTK_OPTION_MENU(con_opt), con_menu);
    gtk_box_pack_start(GTK_BOX(vbox), con_opt, FALSE, FALSE, 0);

    /* connected label */
    connlabel = gtk_label_new(_("Not connected"));
    gtk_box_pack_start(GTK_BOX(vbox), connlabel, FALSE, FALSE, 0);

    /* sent/received labels */
    vbox = gtk_vbox_new(FALSE, 6);
    gtk_table_attach_defaults(GTK_TABLE(table), vbox, 0, 2, 2, 4);

    output.label = gtk_label_new("");
    gtk_box_pack_start(GTK_BOX(vbox), output.label, FALSE, TRUE, 0);

    input.label = gtk_label_new("");
    gtk_box_pack_start(GTK_BOX(vbox), input.label, FALSE, TRUE, 0);

    /* dial / hangup buttons */
    vbox = gtk_vbox_new(FALSE, 6);
    gtk_table_attach_defaults(GTK_TABLE(table), 
			      vbox, 1, 2, 0, 2);

    dialbutton = gtk_button_new_with_label(_("Dial"));
    g_signal_connect (GTK_OBJECT (dialbutton), "clicked",
                        GTK_SIGNAL_FUNC(menu_dial), window);
    gtk_box_pack_start(GTK_BOX(vbox), dialbutton, FALSE, FALSE, 0);

    hangbutton = gtk_button_new_with_label(_("Hang up"));
    g_signal_connect (GTK_OBJECT (hangbutton), "clicked",
                        GTK_SIGNAL_FUNC(menu_hangup), window);
    gtk_box_pack_start(GTK_BOX(vbox), hangbutton, FALSE, FALSE, 0);

    /* finally net load meters */
    vbox = gtk_vbox_new(FALSE, 0);
    gtk_table_attach_defaults(GTK_TABLE(table),
			      vbox, 2, 3, 0, 4);

    label = gtk_label_new(_("output"));
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

    netload_pixmap_new(frame, &output, NETLOAD_WIDTH, NETLOAD_HEIGHT);
    
    label = gtk_label_new(_("input"));
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

    netload_pixmap_new(frame, &input, NETLOAD_WIDTH, NETLOAD_HEIGHT);

    /* Setup initial widget states */
    set_dial_sensitive(FALSE, FALSE);
    set_connection_sensitive(FALSE);

    if (autoconnect || autodial_isp != NULL) menu_connect();

    gtk_widget_show_all(mainvbox);

#ifdef BUILD_APPLET
    if (!applet_running)		/* mainwin only */
	gtk_widget_show(window);
    else{

	/* apply preference selection first */	    
	if (show_mainwin_at_start)
	    applet_showmain();
        else
            applet_hidemain();

	/* override preference with cmdline selection if required */
	if (!main_running)
            applet_hidemain();
    }
#else
    gtk_widget_show(window);
#endif

    gtk_widget_hide(input.label);
    gtk_widget_hide(output.label);
    //gtk_widget_hide(anim_da);

#ifdef BUILD_APPLET
    if (applet_running)
        applet_widget_gtk_main();
    else
#endif

    gtk_main ();

    return 0;
}
