/*
 serverinfo.c : GMasqDialer

    Copyright (C) 1998-1999 Timo Sirainen
    Copyright (C) 2000-2002 David Pinson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gmasqdialer.h"

static gchar *version, *author;
static glong ccount;
static GList *wholist;
static gint stimetag;

static GtkWidget *license_dialog = NULL, *history_dialog = NULL;

//static GtkWidget *gtkwholist;
static GtkListStore *store_wholist;

static GtkWidget *ccountlabel;

/* History / License dialog things */
static void license_delete_event(void)
{
    gtk_widget_destroy(license_dialog);
    license_dialog = NULL;
}

static void history_delete_event(void)
{
    gtk_widget_destroy(history_dialog);
    history_dialog = NULL;
}

static void handle_textfile(CALLBACK_DATA *data, 
			    gchar *title, 
			    GtkWidget **dialog, 
			    GtkSignalFunc closefunc)
{
    GtkWidget *text;
    GtkTextBuffer *buffer;
    GtkWidget *hbox;
    GtkWidget *sbar;
#ifndef HAVE_GNOME
    GtkWidget *button;
#endif

    switch (data->u.list.type)
    {
        case LIST_START:

	    *dialog = gtk_hig_dialog_new_with_buttons( title, 
						       NULL,
						       GTK_DIALOG_DESTROY_WITH_PARENT, 
						       GTK_STOCK_CLOSE, 
						       GTK_RESPONSE_NONE,
						       NULL);
    
	    g_signal_connect_swapped (GTK_OBJECT (*dialog), "response", 
				      G_CALLBACK (closefunc),
				      GTK_OBJECT (*dialog));


	    gtk_dialog_set_default_response (GTK_DIALOG(*dialog),
					     GTK_RESPONSE_CLOSE);

            gtk_container_set_border_width(GTK_CONTAINER(*dialog), 6);

            gtk_widget_set_usize(*dialog, 550, 450);

            gtk_window_set_policy(GTK_WINDOW(*dialog), TRUE, TRUE, FALSE);

            g_signal_connect(GTK_OBJECT(*dialog), 
			       "delete_event", 
			       closefunc, NULL);

            hbox = gtk_hbox_new(FALSE, 0);
            gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(*dialog)->vbox),
			       hbox, TRUE, TRUE, 6);

            //text = gtk_text_new(NULL, NULL);
	    text = gtk_text_view_new();
	    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(text));
            //gtk_text_set_editable(GTK_TEXT(text), FALSE);
            gtk_box_pack_start(GTK_BOX(hbox), text, TRUE, TRUE, 0);

            //sbar = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
            //gtk_box_pack_start(GTK_BOX(hbox), sbar, FALSE, FALSE, 0);

            //gtk_text_freeze(GTK_TEXT(text));
            break;

        case LIST_END:
            //gtk_text_thaw(GTK_TEXT(text));
            gtk_widget_show_all(*dialog);
            break;

        case LIST_ITEM:
            //gtk_text_insert(GTK_TEXT(text), 
	    //		    NULL, NULL, NULL, data->u.list.data, -1);
            //gtk_text_insert(GTK_TEXT(text), 
	    //	    NULL, NULL, NULL, "\n", 1);
	    gtk_text_buffer_set_text(buffer, data->u.list.data, -1);
	    gtk_text_buffer_set_text(buffer, "\n", 1);
            break;
    }
}

static void callback_license(CALLBACK_DATA *data)
{
    if (data->status == 0) return;

    handle_textfile(data, 
		    _("Server license"), 
		    &license_dialog, 
		    license_delete_event);
}

static void callback_history(CALLBACK_DATA *data)
{
    if (data->status == 0) return;

    handle_textfile(data, 
		    _("Server history"), 
		    &history_dialog, 
		    history_delete_event);
}

static void show_license(void)
{
    if (license_dialog == NULL)
        mserver_license(callback_license);
    else
        gdk_window_raise(license_dialog->window);
}

static void show_history(void)
{
    if (history_dialog == NULL)
        mserver_history(callback_history);
    else
        gdk_window_raise(history_dialog->window);
}

/* Client gtkwholist things */
static void parse_who(CALLBACK_DATA *data)
{
    char *tmp, *prev;

    for (prev = tmp = data->u.text; *tmp != '\0'; tmp++)
    {
        if (*tmp == ':')
        {
            *tmp++ = '\0';
            wholist = g_list_append(wholist, g_strdup(prev));
            prev = tmp;
        }
    }
    wholist = g_list_append(wholist, g_strdup(prev));
}

static void create_wholist(void)
{
    GList *tmp;
    //GtkWidget *li;
    GtkTreeIter iter;

    gtk_list_store_append(GTK_LIST_STORE(store_wholist), &iter);

    for (tmp = g_list_first(wholist); tmp != NULL; tmp = tmp->next)
    {
        //li = gtk_list_item_new_with_label(tmp->data);
	gtk_list_store_set(GTK_LIST_STORE(store_wholist), &iter,
			   0, tmp->data,
			   -1);
        g_free(tmp->data); 
	//tmp->data = li;
    }
    //gtk_list_append_items(GTK_LIST(gtkwholist), wholist);
    //gtk_widget_show_all(gtkwholist);
}

static void serverinfo_who(CALLBACK_DATA *data)
{
    if (data->status == 0) return;

    parse_who(data);
    create_wholist();
}

static void serverinfo_ccount(CALLBACK_DATA *data)
{
    GString *str;

    if (data->status == 0 || ccount == data->u.digit) return;

    ccount = data->u.digit;

    str = g_string_new(NULL);
    g_string_printf(str, _("%ld clients connected from hosts:"), ccount);
    gtk_label_set_text(GTK_LABEL(ccountlabel), str->str);
    g_string_free(str, TRUE);

    //gtk_list_clear_items(GTK_LIST(gtkwholist), 0, -1);
    printf("ATTN: write a list clear bit of code: %s %d\n", __FILE__,__LINE__);

    wholist = NULL;
    mserver_listclients(serverinfo_who);
}

static int serverinfo_timefunc(void)
{
    mserver_clientcount(serverinfo_ccount);
    return 1;
}

static void delete_event(GtkWidget *dialog)
{
    gtk_widget_destroy(dialog);
    gtk_timeout_remove(stimetag);
}

static void server_dialog(void)
{
    GtkWidget *dialog;
    GtkWidget *vbox, *hbox, *sbox, *frame;
    GtkWidget *main_hbox, *main_vbox;
    GtkWidget *button;
    GtkWidget *label;
    GString *str;

    GtkWidget *tree_wholist;

    str = g_string_new(NULL);

    dialog = gtk_hig_dialog_new_with_buttons(_("Server Info"),
					     NULL,
					     GTK_DIALOG_DESTROY_WITH_PARENT,
					     GTK_STOCK_CLOSE, 
					     GTK_RESPONSE_CLOSE,
					     NULL);

    gtk_dialog_set_default_response (GTK_DIALOG(dialog),
				     GTK_RESPONSE_CLOSE);

    g_signal_connect_swapped (GTK_OBJECT (dialog), "response", 
			      G_CALLBACK (delete_event),
			      GTK_OBJECT (dialog));


    /* try to get HIG layout... */
    main_hbox = gtk_hbox_new(FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER(main_hbox), 6);

    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
		       main_hbox, FALSE, FALSE, 0);

    main_vbox = gtk_vbox_new(FALSE, 0);

    gtk_box_pack_start(GTK_BOX(main_hbox), main_vbox, FALSE, FALSE, 0);

    /* Label */
    label = gtk_label_new(version);
    gtk_box_pack_start(GTK_BOX(main_vbox), label, FALSE, FALSE, 0);

    g_string_printf(str, _("Author: %s"), author);
    label = gtk_label_new(str->str);
    gtk_box_pack_start(GTK_BOX(main_vbox), label, FALSE, FALSE, 0);

    /* license & history buttons */
    hbox = gtk_hbox_new(TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER(hbox), 6);
    gtk_box_set_spacing (GTK_BOX (hbox), 12);

    gtk_box_pack_start(GTK_BOX(main_vbox), hbox, TRUE, TRUE, 0);

    button = gtk_button_new_with_label(_("Show license"));
    g_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(show_license), NULL);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);

    button = gtk_button_new_with_label(_("Show history"));
    g_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(show_history), NULL);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);

    /* List of connected hosts */
    g_string_printf(str, _("%ld clients connected from hosts:"), ccount);
    ccountlabel = gtk_label_new(str->str);
    gtk_box_pack_start(GTK_BOX(main_vbox), ccountlabel, FALSE, TRUE, 0);


    sbox = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sbox),
                                   GTK_POLICY_AUTOMATIC, 
				   GTK_POLICY_AUTOMATIC);

    gtk_box_pack_start(GTK_BOX(main_vbox), sbox, FALSE, FALSE, 0);

    //gtkwholist = gtk_list_new();
    store_wholist = gtk_list_store_new(1, G_TYPE_STRING);
    tree_wholist = gtk_tree_view_new();

    //gtk_list_set_selection_mode(GTK_LIST(gtkwholist), GTK_SELECTION_BROWSE);
    //gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sbox), 
    //				  gtkwholist);

    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sbox), 
					  tree_wholist);

    create_wholist();

    stimetag = gtk_timeout_add(1000, (GtkFunction) serverinfo_timefunc, NULL);

    gtk_widget_show_all(dialog);
    g_free(author); g_free(version);
    g_string_free(str, TRUE);
}

static void callback_author(CALLBACK_DATA *data)
{
    if (data->status == 0)
    {
        g_free(version);
        return;
    }

    author = data->u.text;
    server_dialog();
}

static void callback_version(CALLBACK_DATA *data)
{
    if (data->status == 0) return;

    version = data->u.text;
}

static void callback_who(CALLBACK_DATA *data)
{
    if (data->status == 0) return;

    parse_who(data);
}

static void callback_ccount(CALLBACK_DATA *data)
{
    if (data->status == 0) return;

    ccount = data->u.digit;
    wholist = NULL;
}

void menu_serverinfo(void)
{
    if (!mserver_connected()) return;

    mserver_clientcount(callback_ccount);
    mserver_listclients(callback_who);
    mserver_version(callback_version);
    mserver_author(callback_author);
}
